#pragma once

#include <vector>
#include <database/SqlPreparedQuery.h>
#include <database/export.h>

struct sqlite3_stmt;

namespace pdp {

class SqliteDatabase;

/**
 * Sqlite implementation of the SqlPreparedQuery class. Since this class
 * implements a template pattern, all the publicly accessible functions are
 * defined in the base class.
 *
 * @ingroup database
 */
class PDPDATABASE_EXPORT SqlitePreparedQuery : public SqlPreparedQuery
{
public:
  /**
   * Constructs the PreparedQuery for the given database and with the given SQL
   * statement.  The statement isn't compiled until Prepare() is called either
   * directly or when bindAndExecute is called for the first time.  
   *
   * Lifetime of the object is expected to be controlled by the QObject parent
   * passed in. The parent should make sure these are deleted when the database
   * is disconnected.  Users of these queries shouldn't attempt to free the
   * memory directly.
   */
  explicit SqlitePreparedQuery(
    SqliteDatabase* db,
    const std::string& statement);

  /**
   * SqlitePreparedQuery object destructor.
   */
  virtual ~SqlitePreparedQuery();

protected:

  // Documented in the base class
  virtual bool doPrepare();

  virtual bool doBindAndExecute(
    const SqlPreparedQuery::BindList& binds, 
    pdp::SqlResult& data);

private:
  SqliteDatabase* m_db;
  std::vector<QueryRow> m_results;
  sqlite3_stmt* m_statement;
};

} // namespace pdp

