#pragma once

#include <vector>
#include <database/SqlResult.h>
#include <database/SqlQuery.h>
#include <database/export.h>

namespace pdp {

class SqliteDatabase;
class SqlitePreparedQuery;

/**
 * Sqlite implementation of the database Query class
 *
 * @ingroup database
 */
class PDPDATABASE_EXPORT SqliteQuery : public SqlQuery
{
public:
  /**
   * SqliteQuery object constructor.
   */
  SqliteQuery(
    SqliteDatabase& db);

  /**
   * SqliteQuery object destructor.
   */
  virtual ~SqliteQuery();

  // virtual functions documented in the base class
  virtual bool exec(
    const std::string& sqlCommand) override;

  virtual bool next() const override;

  virtual const SqlValue& value(
    int idx) const override;

  virtual const SqlValue& value(
  	unsigned int row,
    unsigned int col) const override;

  virtual unsigned int numRows() const override;

  virtual unsigned int numColumns() const override;

  virtual unsigned int rowsAffected() const override;

  virtual void clear() override;

  virtual const std::string errorMessage() const override;

private:
  SqliteDatabase& m_db;
  SqlitePreparedQuery* m_prepared;
  SqlResult m_data;
};

} // namespace pdp

