namespace pdp
{

template<unsigned int N, typename E>
LineSegmentR<N, E>::LineSegmentR(
  const VectorR<N, E>& start,
  const VectorR<N, E>& end)
  : m_start(start),
    m_end(end)
{
}

template<unsigned int N, typename E>
LineSegmentR<N, E>::LineSegmentR(
  const LineSegmentR<N, E>& l)
  : m_start(l.m_start),
    m_end(l.m_end)
{
}

template<unsigned int N, typename E>
LineSegmentR<N, E>& LineSegmentR<N, E>::operator=(
  const LineSegmentR<N, E>& l)
{
  m_start = l.start();
  m_end = l.end();
  return *this;
}

template<unsigned int N, typename E>
bool LineSegmentR<N, E>::operator==(
  const LineSegmentR& l) const
{
  return (m_start == l.start() && m_end == l.end());
}

template<unsigned int N, typename E>
std::ostream& operator<<(
  std::ostream& out,
  const LineSegmentR<N, E> & l)
{
  out << "start: " << l.start() << " end:" << l.end();
  return out;
}

template<unsigned int N, typename E>
double length(
	const LineSegmentR<N, E>& l)
{
  VectorR<N, E> span;
  sub(l.end(), l.start(), span);
  return length(span);
}

template<typename E>
void distanceFromPointToLineSegment(
	const VectorR<2, E>& p,
  const LineSegmentR<2, E>& l,
  double& segmentDistance,
  double& lineDistance)
{
  double r_numerator = (double)((p[X]- l.start()[X]) * (l.end()[X] - l.start()[X]) +
  	(p[Y]- l.start()[Y]) * (l.end()[Y] - l.start()[Y]));
  double r_denomenator = (double)((l.end()[X] - l.start()[X]) * (l.end()[X] - l.start()[X]) +
  	(l.end()[Y] - l.start()[Y]) * (l.end()[Y] - l.start()[Y]));
  double r = r_numerator / r_denomenator;
  double px = (double)l.end()[X] + r * (double)(l.end()[X] - l.start()[X]);
  double py = (double)l.end()[Y] + r * (double)(l.end()[Y] - l.start()[Y]);
  double s =  (double)((l.start()[Y] - p[Y]) * (l.end()[X] - l.start()[X]) -
  	(l.start()[X] - p[X]) * (l.end()[Y] - l.start()[Y])) / r_denomenator;
  lineDistance = std::fabs(s) * sqrt(r_denomenator);
  double xx = px;
  double yy = py;
  if ((r >= 0) && (r <= 1))
  {
    segmentDistance = lineDistance;
  }
  else
  {
    double dist1 = (double)((p[X]- l.start()[X]) * (p[X]- l.start()[X]) +
    	(p[Y]- l.start()[Y]) * (p[Y]- l.start()[Y]));
    double dist2 = (double)((p[X]- l.end()[X]) * (p[X]- l.end()[X]) +
    	(p[Y]- l.end()[Y]) * (p[Y]- l.end()[Y]));
    if (dist1 < dist2)
    {
      xx = (double)l.start()[X];
      yy = (double)l.start()[Y];
      segmentDistance = sqrt(dist1);
    }
    else
    {
      xx = (double)l.end()[X];
      yy = (double)l.end()[Y];
      segmentDistance = sqrt(dist2);
    }
  }
  return;
}

// see   http://mathworld.wolfram.com/Point-LineDistance3-Dimensional.html
template<unsigned int N, typename E>
void projectPointToLineSegment(
  const VectorR<N, E>& x0,
  const LineSegmentR<N, E>& lineSeg,
  VectorR<N, E>& pProjection,
  double& t)
{

  VectorR<N, E> v1;
  sub(lineSeg.start(), x0, v1);

  VectorR<N, E> v2;
  sub(lineSeg.end(), lineSeg.start(), v2);

  t = -(dot(v1, v2)) / dot(v2, v2);

  pProjection = lineSeg.start() + v2 * (E) t;

}

template<unsigned int N, typename E>
bool almostEqual(
  const LineSegmentR<N, E>& l0,
  const LineSegmentR<N, E>& l1,
  double epsilon)
{
  return (almostEqual(l0.start(), l1.start()) && almostEqual(l0.end(), l1.end()));
}

} // namespace pdp
