namespace pdp {

template<unsigned int N, typename E>
TriangleR<N, E>::TriangleR(
  const VectorR<N, E>& p0,
  const VectorR<N, E>& p1,
  const VectorR<N, E>& p2,
  const VectorR<N, E>& n0,
  const VectorR<N, E>& n1,
  const VectorR<N, E>& n2)
  : m_p0(p0),
    m_p1(p1),
    m_p2(p2),
    m_n0(n0),
    m_n1(n1),
    m_n2(n2)
{
}

template<unsigned int N, typename E>
TriangleR<N, E>::TriangleR(
  const TriangleR<N, E>& t)
  : m_p0(t.m_p0),
    m_p1(t.m_p1),
    m_p2(t.m_p2),
    m_n0(t.m_n0),
    m_n1(t.m_n1),
    m_n2(t.m_n2)
{
}

template<unsigned int N, typename E>
void TriangleR<N, E>::computeNormal()
{
  VectorR<N, E> v0, v1, n;
  sub(m_p2, m_p0, v0);
  sub(m_p1, m_p0, v1);
  cross(v0, v1, n);
  normalize(n);
  m_n0 = n;
  m_n1 = n;
  m_n2 = n;
}

template<unsigned int N, typename E>
double TriangleR<N, E>::computeArea() const
{
  VectorR<N, E> v0, v1, n;
  sub(m_p2, m_p0, v0);
  sub(m_p1, m_p0, v1);
  cross(v0, v1, n);

  return 0.5 * length(n);
}

template<unsigned int N, typename E>
TriangleR<N, E>& TriangleR<N, E>::operator=(
  const TriangleR<N, E>& t)
{
  m_p0 = t.p0();
  m_p1 = t.p1();
  m_p2 = t.p2();
  m_n0 = t.n0();
  m_n1 = t.n1();
  m_n2 = t.n2();
  return* this;
}

template<unsigned int N, typename E>
bool TriangleR<N, E>::operator==(
  const TriangleR& t) const
{
  return (m_p0 == t.p0() && m_p1 == t.p1() && m_p2 == t.p2() &&
    m_n0 == t.n0() && m_n1 == t.n1() && m_n2 == t.n2());
}

template<unsigned int N, typename E>
std::ostream& operator<<(
  std::ostream& out,
  const TriangleR<N, E> & t)
{
  out << "p0: " << t.p0() << " p1:" << t.p1() << " p2:" << t.p2() << " n0:" << t.n0() << " n1:" << t.n1() << " n2:" << t.n2();
  return out;
}

template<unsigned int N, typename E>
bool almostEqual(
  const TriangleR<N, E>& t0,
  const TriangleR<N, E>& t1,
  double epsilon)
{
  return (almostEqual(t0.p0(), t1.p0()) && almostEqual(t0.p1(), t1.p1()) && almostEqual(t0.p2(), t1.p2()) &&
    almostEqual(t0.n0(), t1.n0()) && almostEqual(t0.n1(), t1.n1()) && almostEqual(t0.n2(), t1.n2()));
}

} // namespace pdp
