/** @file
 *
 * Robot manager Interface
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2019
 */
#pragma once

#include <models/export.h>
#include <models/Robot.h>
#include <models/Crazyflie.h>
#include <models/CrazyflieManager.h>
#include <models/TelloManager.h>

using boost::asio::ip::udp;

namespace pdp {

class PDPMODELS_EXPORT RobotFactory
{
public:
  enum RobotType
  {
    RobotTypeNone = 0,
    RobotTypeCrazyflie,
    RobotTypeTello,
    RobotTypeNao
  };

  /**
   * Use example:
   * Robot* robot = RobotFactory::createRobot(RobotFactory::RobotTypeTello);
   */
  static Robot* createRobot(
    RobotType robotType,
    std::function<void()>* commandResponseFunction,
    std::function<void(std::string)>* stateUpdateFunction,
    std::function<void(const unsigned char*, int, int)>* frameDisplayFunction,
    std::function<void()>* commandsTimeoutFunction,
    std::function<void()>* stateTimeoutFunction,
    std::function<void()>* videoTimeoutFunction)
  {
    Robot* robot = nullptr;
    switch(robotType)
    {
    case RobotTypeNone:
      break;
    case RobotTypeCrazyflie:
      robot = new CrazyflieManager(commandResponseFunction, stateUpdateFunction, frameDisplayFunction,
        commandsTimeoutFunction, stateTimeoutFunction, videoTimeoutFunction);
      break;
    case RobotTypeTello:
      robot = new TelloManager(commandResponseFunction, stateUpdateFunction, frameDisplayFunction,
        commandsTimeoutFunction, stateTimeoutFunction, videoTimeoutFunction);
      break;
    case RobotTypeNao:
      break;
    }
    return robot;
  }
};

} // namespace pdp
