/** @file
 *
 * QT based opengl widget
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <QPixmap>
#include <QWidget>
#include <QMouseEvent>
#include <elements/VectorR.h>
#include <imaging/ImageR2.h>
#include <render/MouseEvent.h>
#include <render/ImageRenderingEngine.h>
#include <qtElements/export.h>

class PDPQTELEMENTS_EXPORT ImageWidget : public QWidget
{
Q_OBJECT

public:
  ImageWidget(
    pdp::ImageRenderingEngine* engine,
    QWidget* parent);

  ~ImageWidget();

  void render(
    pdp::Image2dDetectionsAdaptor::ClassificationType classificationType,
    pdp::ImageR2::StructureChannel structureChannel,
    const std::vector<pdp::Color>& classColors,
    const std::vector<bool>& showClasses,
    const pdp::VectorR2f& displaySample);

  void renderBuffer(
    const pdp::VectorR2f& displaySample);

protected:
  virtual void paintEvent(
    QPaintEvent* paintEvent) override;

  virtual void mousePressEvent(
    QMouseEvent* event) override;

  virtual void mouseReleaseEvent(
    QMouseEvent* event) override;

  virtual void mouseMoveEvent(
    QMouseEvent *event) override;

  virtual void mouseDoubleClickEvent(
    QMouseEvent* event) override;

private:
  pdp::VectorR2i mousePosition(
    QMouseEvent* event);

  pdp::MouseEvent::ButtonState mouseButton(
    Qt::MouseButtons state);

  pdp::MouseEvent::ModifierKeyState modifierKeys(
    Qt::KeyboardModifiers state);

  void updateRender();

private:
  std::function<void()> m_updateRender;

  // Render function specific parameter variables
  pdp::Image2dDetectionsAdaptor::ClassificationType m_classificationType;
  pdp::ImageR2::StructureChannel m_structureChannel;
  std::vector<bool> m_showClasses;

  // Render state variables
  std::vector<pdp::Color> m_colorLookupTable;
  pdp::VectorR2f m_displaySample;
  bool m_renderNeedsUpdate;
  bool m_renderBufferNeedsUpdate;
  QPixmap m_pixmap;
  pdp::ImageRenderingEngine* m_engine;
};
