/** @file
 *
 * QT based opengl widget
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <QPixmap>
#include <QWidget>
#include <QMouseEvent>
#include <elements/VectorR.h>
#include <imaging/VolumeR3.h>
#include <render/MouseEvent.h>
#include <render/VolumeRenderingEngine.h>
#include <qtElements/export.h>

class PDPQTELEMENTS_EXPORT VolumeWidget : public QWidget
{
Q_OBJECT

public:
  VolumeWidget(
    pdp::VolumeRenderingEngine* engine,
    QWidget* parent);

  ~VolumeWidget();

  void render(
    pdp::VolumeRenderingEngine::LayoutOptions layout,
    pdp::VolumeR3::DatasetType datasetType,
    pdp::VolumeR3::VolumeChannel channel,
    pdp::VolumeR3::RenderOrientation renderOrientation,
    pdp::VolumeR3::StructureType structureType,
    float structureAlpha,
    bool useNavigationAgent);

protected:
  virtual void paintEvent(
    QPaintEvent* paintEvent) override;

  virtual void mousePressEvent(
    QMouseEvent* event) override;

  virtual void mouseReleaseEvent(
    QMouseEvent* event) override;

  virtual void mouseMoveEvent(
    QMouseEvent *event) override;

  virtual void mouseDoubleClickEvent(
    QMouseEvent* event) override;

private:
  pdp::VectorR2i mousePosition(
    QMouseEvent* event);

  pdp::MouseEvent::ButtonState mouseButton(
    Qt::MouseButtons state);

  pdp::MouseEvent::ModifierKeyState modifierKeys(
    Qt::KeyboardModifiers state);

  void updateRender();

private:
  std::function<void()> m_updateRender;
  pdp::VolumeRenderingEngine::LayoutOptions m_layout;
  pdp::VolumeR3::DatasetType m_datasetType;
  pdp::VolumeR3::VolumeChannel m_channel;
  pdp::VolumeR3::RenderOrientation m_renderOrientation;
  pdp::VolumeR3::StructureType m_structureType;
  float m_structureAlpha;
  bool m_useNavigationAgent;
  QPixmap m_pixmap;
  pdp::VolumeRenderingEngine* m_engine;
};
