/** @file
 *
 * Base64 encoding and decoding functions
 *
 * @author Paul Pantalone
 * @copyright Paul Pantalone. All Rights Reserved
 * @date 2018
 */
#pragma once

#include <vector>
#include <string>
#include <utilities/export.h>

namespace pdp
{
/**
 * Creates a base64 encoded string from the input byte array
 *
 * @param bytes_to_encode Input byte array to encode
 * @param in_len Length of the input character byte array in number of bytes
 * @return Encoded base64 string version of the input byte array
 *
 * @ingroup utilities
 */
PDPUTILITIES_EXPORT std::string base64_encode(
  const unsigned char* bytes_to_encode,
  unsigned int in_len);

/**
 * Creates a standard string by decoding the input base64 encoded string
 *
 * @param encoded_string Input base64 encoded string
 * @return Standard string created from decoding the input base64 encoded string
 *
 * @ingroup utilities
 */
PDPUTILITIES_EXPORT std::string base64_decode(
  const std::string& encoded_string);

PDPUTILITIES_EXPORT std::vector<unsigned char> base64DecodeToUBytes(
  std::string const& encoded_string);

} // namespace pdp
